@@ -9,6 +9,7 @@
 
 #include <treelite/base.h>
 #include <treelite/version.h>
+#include <treelite/optional.h>
 #include <algorithm>
 #include <map>
 #include <memory>
@@ -451,6 +452,17 @@ class Tree {
       // Use unsafe access here, since we may need to take the address of one past the last
       // element, to follow with the range semantic of std::vector<>.
   }
+  /*!
+   * \brief Get the largest category value used in all categorical splits in this tree.
+   *        If there are no categorical splits, returns null.
+   */
+  inline optional<uint32_t> MaxCategory() const {
+    if (matching_categories_.Empty()) {
+      return optional<uint32_t>{};
+    }
+    return optional<uint32_t>{*std::max_element(matching_categories_.Data(),
+                                                matching_categories_.End())};
+  }
   /*!
    * \brief get feature split type
    * \param nid ID of node being queried
@@ -612,6 +624,14 @@ struct ModelParam {
    * It must be strictly positive; if unspecified, it is set to 1.0.
    */
   float sigmoid_alpha;
+  /*!
+   * \brief scaling parameter for exponential standard ratio transformation
+   * `expstdratio(x) = exp2(-x / c)`
+   *
+   * This parameter is used only when `pred_transform` is set to `'exponential_standard_ratio'`.
+   * If unspecified, it is set to 1.0.
+   */
+  float ratio_c;
   /*!
    * \brief global bias of the model
    *
@@ -621,7 +641,7 @@ struct ModelParam {
   float global_bias;
   /*! \} */
 
-  ModelParam() : sigmoid_alpha(1.0f), global_bias(0.0f) {
+  ModelParam() : sigmoid_alpha(1.0f), ratio_c(1.0f), global_bias(0.0f) {
     std::memset(pred_transform, 0, TREELITE_MAX_PRED_TRANSFORM_LENGTH * sizeof(char));
     std::strncpy(pred_transform, "identity", sizeof(pred_transform));
   }
